@@ -160,15 +160,15 @@ static int sg_init_enumerator(burn_drive_enumerator_t *idx_)
 	struct burn_drive_enumeration_state *idx;
 	int bufsize;
 
-	idx = malloc(sizeof(*idx));
+	idx = calloc(1, sizeof(*idx));
 	if (idx == NULL) {
-		warnx("can't malloc memory for enumerator");
+		warnx("cannot allocate memory for enumerator");
 		return -1;
 	}
 	idx->skip_device = 0;
 
 	if ((idx->fd = open(XPT_DEVICE, O_RDWR)) == -1) {
-		warn("couldn't open %s", XPT_DEVICE);
+		warn("could not open %s", XPT_DEVICE);
 		free(idx);
 		idx = NULL;
 		return -1;
@@ -183,9 +183,9 @@ static int sg_init_enumerator(burn_drive_enumerator_t *idx_)
 	idx->ccb.ccb_h.func_code = XPT_DEV_MATCH;
 	bufsize = sizeof(struct dev_match_result) * 100;
 	idx->ccb.cdm.match_buf_len = bufsize;
-	idx->ccb.cdm.matches = (struct dev_match_result *)malloc(bufsize);
+	idx->ccb.cdm.matches = (struct dev_match_result *) calloc(1, bufsize);
 	if (idx->ccb.cdm.matches == NULL) {
-		warnx("can't malloc memory for matches");
+		warnx("cannot allocate memory for matches");
 		close(idx->fd);
 		free(idx);
 		return -1;
@@ -487,9 +487,9 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.close_session = mmc_close_session;
 	out.close_track_session = mmc_close;
 	out.read_buffer_capacity = mmc_read_buffer_capacity;
-	out.idata = malloc(sizeof(struct burn_scsi_inquiry_data));
+	out.idata = calloc(1, sizeof(struct burn_scsi_inquiry_data));
 	out.idata->valid = 0;
-	out.mdata = malloc(sizeof(struct scsi_mode_data));
+	out.mdata = calloc(1, sizeof(struct scsi_mode_data));
 	out.mdata->valid = 0;
 	if (out.idata == NULL || out.mdata == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020108,
@@ -759,13 +759,17 @@ int sg_release(struct burn_drive *d)
 
 int sg_issue_command(struct burn_drive *d, struct command *c)
 {
-	int done = 0, err, sense_len = 0, ret, ignore_error, no_retry = 0;
-	int cam_pass_err_recover = 0;
+	int done = 0, err, sense_len = 0, ret, ignore_error, no_retry = 0, i;
+	int cam_pass_err_recover = 0, key, asc, ascq, timeout_ms;
 	union ccb *ccb;
 	char buf[161];
 	static FILE *fp = NULL;
+	time_t start_time;
 
-	snprintf(buf, sizeof (buf), "sg_issue_command  d->cam=%p d->released=%d",
+#define Libburn_use_scsi_eval_cmd_outcomE yes
+
+	snprintf(buf, sizeof (buf),
+		"sg_issue_command  d->cam=%p d->released=%d",
 		(void*)d->cam, d->released);
 	mmc_function_spy(NULL, buf);
 
@@ -854,7 +858,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		ccb->csio.dxfer_len = 0;
 	}
 
-	do {
+	start_time = time(NULL);
+	timeout_ms = 200000;
+	for (i = 0; !done; i++) {
 		memset(c->sense, 0, sizeof(c->sense));
 		err = cam_send_ccb(d->cam, ccb);
 
@@ -867,8 +873,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			if (sense_len > sizeof(c->sense))
 				sense_len = sizeof(c->sense);
 			memcpy(c->sense, &ccb->csio.sense_data, sense_len);
-			if (sense_len >= 14 && cam_pass_err_recover &&
-			    (c->sense[2] & 0x0f))
+			spc_decode_sense(c->sense, sense_len,
+							&key, &asc, &ascq);
+			if (sense_len >= 14 && cam_pass_err_recover && key)
 				ignore_error = 1;
 		}
 
@@ -886,6 +893,7 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				fprintf(stderr, "libburn_EXPERIMENTAL: Emulating [2,3A,00] MEDIUM NOT PRESENT\n");
 #endif
 
+				c->sense[0] = 0x70; /*Fixed format sense data*/
 				c->sense[2] = 0x02;
 				c->sense[12] = 0x3A;
 				c->sense[13] = 0x00;
@@ -901,6 +909,7 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				fprintf(stderr, "libburn_EXPERIMENTAL: Emulating [2,04,00] LOGICAL UNIT NOT READY,CAUSE NOT REPORTABLE\n");
 #endif
 
+				c->sense[0] = 0x70; /*Fixed format sense data*/
 				c->sense[2] = 0x02;
 				c->sense[12] = 0x04;
 				c->sense[13] = 0x00;
@@ -914,6 +923,7 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				fprintf(stderr, "libburn_EXPERIMENTAL: Emulating [5,24,00] INVALID FIELD IN CDB\n");
 #endif
 
+				c->sense[0] = 0x70; /*Fixed format sense data*/
 				c->sense[2] = 0x05;
 				c->sense[12] = 0x24;
 				c->sense[13] = 0x00;
@@ -944,11 +954,26 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				fprintf(stderr, "libburn_EXPERIMENTAL: CAM_STATUS= %d .Emulating [2,04,00] LOGICAL UNIT NOT READY,CAUSE NOT REPORTABLE\n", (ccb->ccb_h.status & CAM_STATUS_MASK));
 #endif
 
+				c->sense[0] = 0x70; /*Fixed format sense data*/
 				c->sense[2] = 0x02;
 				c->sense[12] = 0x04;
 				c->sense[13] = 0x00;
 				no_retry = 1;
 			}
+
+
+#ifdef Libburn_use_scsi_eval_cmd_outcomE
+
+			/* >>> Need own duration time measurement.
+			       Then remove bit1 from flag.
+			*/
+			done = scsi_eval_cmd_outcome(d, c, fp, c->sense,
+						sense_len, 0, start_time,
+						timeout_ms, i,
+						2 | !!ignore_error);
+
+#else /* Libburn_use_scsi_eval_cmd_outcomE */
+
 			if (no_retry || ignore_error || !c->retry) {
 				c->error = 1;
 				{ret = 1; goto ex;}
@@ -959,8 +984,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				if (burn_sg_log_scsi & 3) {
 					/* >>> Need own duration time
 					       measurement. Then remove bit1 */
-					scsi_log_err(c, fp, c->sense, 0,
-							(c->error != 0) | 2);
+					scsi_log_err(c, fp, c->sense,
+					    sense_len > 0 ? sense_len : 18,
+					    0, 1 | 2);
 					scsi_log_cmd(c,fp,0);
 				}
 				break;
@@ -968,19 +994,36 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				done = 1;
 				c->error = 1;
 				break;
+			case GO_ON:
+				if (burn_sg_log_scsi & 3)
+					/* >>> Need own duration time
+					       measurement. Then remove bit1 */
+					scsi_log_err(c, fp, c->sense,
+					    sense_len > 0 ? sense_len : 18,
+					    0, 1 | 2);
+				{ret = 1; goto ex;}
 			}
+
+#endif /* ! Libburn_use_scsi_eval_cmd_outcomE */
+
 		} else {
 			done = 1;
 		}
 	} while (!done);
 	ret = 1;
 ex:;
+
+#ifndef Libburn_use_scsi_eval_cmd_outcomE
+
 	if (c->error)
 		scsi_notify_error(d, c, c->sense, 18, 0);
 
 	if (burn_sg_log_scsi & 3)
 		/* >>> Need own duration time measurement. Then remove bit1 */
-		scsi_log_err(c, fp, c->sense, 0, (c->error != 0) | 2);
+		scsi_log_err(c, fp, c->sense, sense_len > 0 ? sense_len : 18,
+						0, (c->error != 0) | 2);
+
+#endif /* ! Libburn_use_scsi_eval_cmd_outcomE */
 
 	cam_freeccb(ccb);
 	return ret;
@@ -996,6 +1039,10 @@ int burn_os_is_2k_seekrw(char *path, int flag)
         struct stat stbuf;
 	char *spt;
 	int i, e;
+#ifdef Libburn_DIOCGMEDIASIZE_ISBLK
+	int fd;
+	off_t add_size;
+#endif
 
         if (stat(path, &stbuf) == -1)
                 return 0;
@@ -1003,6 +1050,22 @@ int burn_os_is_2k_seekrw(char *path, int flag)
                 return 1;
 	if (!S_ISCHR(stbuf.st_mode))
 		return 0;
+
+#ifdef Libburn_DIOCGMEDIASIZE_ISBLK
+
+	/* If it throws no error with DIOCGMEDIASIZE then it is a
+	   'block device'
+	*/
+	fd = open(path, O_RDONLY);
+	if (fd == -1)
+		return 0;
+	ret = ioctl(fd, DIOCGMEDIASIZE, &add_size);
+	close(fd);
+
+	return 1;
+
+#else /* Libburn_DIOCGMEDIASIZE_ISBLK */
+
 	spt = strrchr(path, '/');
 	if (spt == NULL)
 	        spt = path;
@@ -1025,6 +1088,9 @@ int burn_os_is_2k_seekrw(char *path, int flag)
 	if (strncmp(spt, "fla", e) == 0) /* Flash drive */
 		return 1;
 	return 0;
+
+#endif /* ! Libburn_DIOCGMEDIASIZE_ISBLK */
+
 }
 
 
