@@ -1773,36 +1773,6 @@ int sg_release(struct burn_drive *d)
 }
 
 
-#ifdef NIX
-/* <<< now in spc.c as scsi_log_err */
-
-/** logs outcome of a sg command. flag&1 causes an error message */
-static int sg_log_err(struct command *c, FILE *fp, 
-		sg_io_hdr_t *s,
-		int flag)
-{
-      	if(fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
-		if(flag & 1) {
-  			fprintf(fp,
-			"+++ key=%X  asc=%2.2Xh  ascq=%2.2Xh   (%6d ms)\n",
-				s->sbp[2], s->sbp[12], s->sbp[13],s->duration);
-		} else {
-			scsi_show_cmd_reply(c, fp, 0);
-			fprintf(fp,"%6d ms\n", s->duration);
-		}
-		if (burn_sg_log_scsi & 4)
-			fflush(fp);
-	}
-	if (fp == stderr || !(burn_sg_log_scsi & 2))
-		return 1;
-
-	sg_log_err(c, stderr, s, flag);
-	return 1;
-}
-
-#endif /* NIX */
-
-
 /** Sends a SCSI command to the drive, receives reply and evaluates wether
     the command succeeded or shall be retried or finally failed.
     Returned SCSI errors shall not lead to a return value indicating failure.
@@ -1812,13 +1782,19 @@ static int sg_log_err(struct command *c, FILE *fp,
 */
 int sg_issue_command(struct burn_drive *d, struct command *c)
 {
-	int done = 0, no_c_page = 0, usleep_time, i;
+	int done = 0, no_c_page = 0, i;
 	int err;
 	time_t start_time;
 	sg_io_hdr_t s;
 	/* ts A61030 */
 	static FILE *fp= NULL;
 
+#define Libburn_use_scsi_eval_cmd_outcomE yes
+#ifndef Libburn_use_scsi_eval_cmd_outcomE
+	int usleep_time;
+#endif
+
+
 	/* <<< ts A60821
 	   debug: for tracing calls which might use open drive fds */
 	char buf[161];
@@ -1922,6 +1898,32 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			return -1;
 		}
 
+#ifdef NIX
+/* <<< */
+if(0){
+ static int erst= 1;
+ static unsigned char b00_sense[22]= {
+    0x72, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E,
+    0x09, 0x0C, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
+
+ if (erst > 0) {
+   s.sb_len_wr= 22;
+   memcpy(s.sbp, b00_sense, s.sb_len_wr);
+   erst--;
+ }
+}
+/* <<< */
+#endif /* NIX */
+
+
+#ifdef Libburn_use_scsi_eval_cmd_outcomE
+
+                done = scsi_eval_cmd_outcome(d, c, fp, s.sbp, s.sb_len_wr,
+				s.duration, start_time, s.timeout, i, 0);
+
+#else /* Libburn_use_scsi_eval_cmd_outcomE */
+ 
 		if (s.sb_len_wr) {
 			if (!c->retry) {
 				c->error = 1;
@@ -1933,8 +1935,8 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			case RETRY:
 				done = 0;
 				if (burn_sg_log_scsi & 3) {
-					scsi_log_err(c, fp, s.sbp, s.duration,
-								c->error != 0);
+					scsi_log_err(c, fp, s.sbp, s.sb_len_wr,
+						s.duration, 1);
 					scsi_log_cmd(c,fp,0);
 				}
 				break;
@@ -1942,6 +1944,11 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				done = 1;
 				c->error = 1;
 				break;
+			case GO_ON:
+				if (burn_sg_log_scsi & 3)
+					scsi_log_err(c, fp, s.sbp, s.sb_len_wr,
+						s.duration, 1);
+				goto ex;
 			}
 
 			/* ts A90921 :
@@ -1958,14 +1965,28 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		} else {
 			done = 1;
 		}
+
+#endif /* ! Libburn_use_scsi_eval_cmd_outcomE */
+
 	}
 
 	/* ts A61106 */
+
+#ifdef Libburn_use_scsi_eval_cmd_outcomE
+
+	if (s.host_status != Libburn_sg_host_oK || 
+	    (s.driver_status != Libburn_sg_driver_oK && !c->error)) {
+
+#else /* Libburn_use_scsi_eval_cmd_outcomE */
+
 ex:;
 	if (c->error) {
 		scsi_notify_error(d, c, s.sbp, s.sb_len_wr, 0);
 	} else if (s.host_status != Libburn_sg_host_oK || 
 	    s.driver_status != Libburn_sg_driver_oK) {
+
+#endif /* ! Libburn_use_scsi_eval_cmd_outcomE */
+
 		char msg[161];
 
 		sprintf(msg,
@@ -1980,9 +2001,13 @@ ex:;
 				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
 	}
+
+#ifndef Libburn_use_scsi_eval_cmd_outcomE
 	if (burn_sg_log_scsi & 3)
-		/* <<< sg_log_err(c, fp, &s, c->error != 0); */
-		scsi_log_err(c, fp, s.sbp, s.duration, c->error != 0);
+		scsi_log_err(c, fp, s.sbp, s.sb_len_wr,
+						 s.duration, c->error != 0);
+#endif
+
 	return 1;
 }
 
