@@ -385,9 +385,9 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
  		runtime = nwa-150;
 #endif
 
-	sheet = malloc(sizeof(struct cue_sheet));
+	sheet = calloc(1, sizeof(struct cue_sheet));
 
-	/* ts A61009 : react on failures of malloc(), add_cue_sheet()
+	/* ts A61009 : react on failures of calloc(), add_cue_sheet()
 	               type_to_form() */
 	if (sheet == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020111,
