@@ -1192,7 +1192,7 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
    ret= isoburn_read_iso_head(d, lba, &image_size, NULL, 0);
    if(ret<=0)
      {ret= 0; goto failure;}
-   lba= Libisoburn_overwriteable_starT;
+   lba= o->target_iso_head_size / 2048;
    with_enclosure= 1;
    if((flag & 16) && o->emulation_mode == 1) {
      ret= 1;
@@ -1219,7 +1219,7 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
      /* growisofs aligns to 16 rather than 32. Overwriteable TOC emulation
         relies on not accidentially seeing inter-session trash data.
         But one can safely access 16 blocks earlier because a xorriso header
-        would have overwritten with the unused 16 blocks at its start.
+        would have been overwritten with the unused 16 blocks at its start.
         If libisoburn alignment would increase, then this would not be
         possible any more.
      */
