@@ -172,7 +172,8 @@ int ds_read_block(IsoDataSource *src, uint32_t lba, uint8_t *buffer)
 
    if(ret >= 0)
      ret = -1;
-   sprintf(msg, "ds_read_block(%lu) returns %d", (unsigned long) lba, ret);
+   sprintf(msg, "ds_read_block(%lu) returns %lX",
+           (unsigned long) lba, (unsigned long) ret);
    isoburn_msgs_submit(NULL, 0x00060000, msg, 0, "DEBUG", 0);
    return ret; 
  }
