@@ -36,11 +36,15 @@ int isoburn_toc_entry_new(struct isoburn_toc_entry **objpt,
 int isoburn_toc_entry_destroy(struct isoburn_toc_entry **o, int flag);
 
 
-/* Size of target_iso_head which is to be written during
-   isoburn_activate_session()
+/* Minimal size of target_iso_head which is to be written during
+   isoburn_activate_session().
+   Within this size there is everything that is needed for image access with
+   no partition offset. The actual target_iso_head buffer must be larger by
+   the evential partition offset.
 */
 #define Libisoburn_target_head_sizE (32*2048)
 
+
 struct isoburn {
 
 
@@ -104,9 +108,14 @@ struct isoburn {
  int wrote_well;
 
  
- /* Buffered ISO head from media (should that become part of
-    ecma119_read_opts ?) */
- uint8_t target_iso_head[Libisoburn_target_head_sizE];
+ /* ISO head buffer to be filled by write run */
+ int target_iso_head_size;
+ uint8_t *target_iso_head;
+
+ /* The 2k offset which was read from a loaded image.
+ */
+ uint32_t loaded_partition_offset;
+ 
 
  /* Libisofs image context */
  IsoImage *image;
@@ -243,6 +252,12 @@ isoburn_data_source_new(struct burn_drive *d);
 int isoburn_data_source_shutdown(IsoDataSource *src, int flag);
 
 
+/** Check whether the size of target_iso_head matches the given partition
+    offset. Eventually adjust size.
+*/
+int isoburn_adjust_target_iso_head(struct isoburn *o,
+                                   uint32_t offst, int flag);
+
 /**
  * Options for image reading.
    (Comments here may be outdated. API getter/setter function descriptions
@@ -520,6 +535,16 @@ struct isoburn_imgen_opts {
        and timezone 0 */
     char vol_uuid[17];
 
+    /* The number of unclaimed 2K blocks before start of partition 1 as of
+       the MBR in system area. If not 0 this will cause double volume
+       descriptor sets and double tree.
+    */
+    uint32_t partition_offset;
+    /* Partition table parameter: 1 to 63, 0= disabled/default */
+    int partition_secs_per_head; 
+    /* 1 to 255, 0= disabled/default */
+    int partition_heads_per_cyl;
+
 };
 
 
@@ -555,6 +580,9 @@ struct isoburn_imgen_opts {
    written by random access before they were written sequentially.
    In this case, no copy of the session  1 header is maintained and no TOC
    will be possible. Thus writing begins sequentially at LBA 0.
+
+   IMPORTANT: This macro gives the minimal size of an image header. 
+              It has to be enlarged by the eventual partition offset.
 */
 #define Libisoburn_overwriteable_starT \
                                    ((off_t) (Libisoburn_target_head_sizE/2048))
