@@ -146,6 +146,10 @@ size.
 
 */
 
+/* Important: If you add a public API function then add its name to file
+                  libisoburn/libisoburn.ver
+*/
+
 
                           /* API functions */
 
@@ -220,7 +224,7 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libisofs_req_major  0
 #define isoburn_libisofs_req_minor  6
-#define isoburn_libisofs_req_micro 34 
+#define isoburn_libisofs_req_micro 36 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
@@ -264,7 +268,7 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
 */
 #define isoburn_header_version_major  0
 #define isoburn_header_version_minor  6
-#define isoburn_header_version_micro  0
+#define isoburn_header_version_micro  2
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -1169,7 +1173,7 @@ int isoburn_igopt_get_data_start(struct isoburn_imgen_opts *o, int *lba);
 /** Set resp. get parameters "name" and "timestamp" for a scdbackup checksum
     tag. It will be appended to the libisofs session tag if the image starts at
     LBA 0. See isoburn_disc_track_lba_nwa. The scdbackup tag can be used
-    to verify the image by command scdbackup_verify <device> -auto_end.
+    to verify the image by command scdbackup_verify $device -auto_end.
     See scdbackup/README appendix VERIFY for its inner details.
     @since 0.4.4
     @param o          The option set to work on
@@ -1218,20 +1222,59 @@ int isoburn_igopt_set_system_area(struct isoburn_imgen_opts *o,
 int isoburn_igopt_get_system_area(struct isoburn_imgen_opts *o,
                                   char data[32768], int *options);
 
+/** Control production of a second set of volume descriptors (superblock)
+    and directory trees, together with a partition table in the MBR where the
+    first partition has non-zero start address and the others are zeroed.
+    The first partition stretches to the end of the whole ISO image.
+    The additional volume descriptor set and trees will allow to mount the
+    ISO image at the start of the first partition, while it is still possible
+    to mount it via the normal first volume descriptor set and tree at the
+    start of the image resp. storage device.
+    This makes few sense on optical media. But on USB sticks it creates a
+    conventional partition table which makes it mountable on e.g. Linux via
+    /dev/sdb and /dev/sdb1 alike.
+    @since 0.6.2
+    @param opts
+           The option set to be manipulated.
+    @param block_offset_2k
+           The offset of the partition start relative to device start.
+           This is counted in 2 kB blocks. The partition table will show the
+           according number of 512 byte sectors.
+           Default is 0 which causes no second set and trees.
+           If it is not 0 then it must not be smaller than 16.
+    @param secs_512_per_head
+           Number of 512 byte sectors per head. 1 to 63. 0=automatic.
+    @param heads_per_cyl
+           Number of heads per cylinder. 1 to 255. 0=automatic.
+    @return 1 success, <=0 failure
+ */
+int isoburn_igopt_set_part_offset(struct isoburn_imgen_opts  *opts,
+                                  uint32_t block_offset_2k,
+                                  int secs_512_per_head, int heads_per_cyl);
+int isoburn_igopt_get_part_offset(struct isoburn_imgen_opts *opts,
+                                  uint32_t *block_offset_2k,
+                                  int *secs_512_per_head, int *heads_per_cyl);
+
+
 /** Explicitely set the four timestamps of the emerging ISO image.
     Default with all parameters is 0.
     @since 0.5.4
-    ECMA-119 defines the timestamps in the Primary Volume Descriptor as:
+    @param opts
+           The option set to work on
     @param creation_time
+           ECMA-119 Volume Creation Date and Time
            When "the information in the volume was created."
            A value of 0 means that the timepoint of write start is to be used.
     @param modification_time
+           ECMA-119 Volume Modification Date and Time
            When "the informationin the volume was last modified."
            A value of 0 means that the timepoint of write start is to be used.
     @param expiration_time
+           ECMA-119 Volume Expiration Date and Time
            When "the information in the volume may be regarded as obsolete."
            A value of 0 means that the information never shall expire.
     @param effective_time
+           ECMA-119 Volume Effective Date and Time
            When "the information in the volume may be used."
            A value of 0 means that not such retention is intended.
     @param uuid
@@ -1251,6 +1294,7 @@ int isoburn_igopt_get_pvd_times(struct isoburn_imgen_opts *opts,
                       time_t *expiration_time, time_t *effective_time,
                       char uuid[17]);
 
+
 /* ----------------------------------------------------------------------- */
 /*                      End of Options for image generation                */
 /* ----------------------------------------------------------------------- */
@@ -1313,6 +1357,23 @@ int isoburn_set_read_pacifier(struct burn_drive *drive,
                               int (*read_pacifier)(IsoImage*, IsoFileSource*),
                               void *app_handle);
 
+/** Inquire the partition offset of the loaded image. The first 512 bytes of
+    the image get examined whether they bear an MBR signature and a first
+    partition table entry which matches the size of the image. In this case
+    the start address is recorded as partition offset and internal buffers
+    get adjusted.
+    See also isoburn_igopt_set_part_offset().
+    @since 0.6.2
+    @param drive           The drive with the loaded image
+    @param block_offset_2k returns the recognized partition offset
+    @return <0 = error
+             0 = no partition offset recognized
+             1 = acceptable non-zero offset, buffers are adjusted
+             2 = offset is credible but not acceptable for buffer size
+*/ 
+int isoburn_get_img_partition_offset(struct burn_drive *drive,
+                                     uint32_t *block_offset_2k);
+
 
 /** Set the IsoImage to be used with a drive. This eventually releases
     the reference to the old IsoImage attached to the drive.
@@ -1611,3 +1672,7 @@ void isoburn_finish(void);
 int isoburn_needs_emulation(struct burn_drive *d);
  
 
+/* ---------------------------- Test area ----------------------------- */
+
+/* no tests active, currently */
+
