@@ -286,6 +286,12 @@ const char *iso_error_to_msg(int errcode)
         return "Session does not start at LBA 0. scdbackup checksum tag not written.";
     case ISO_BOOT_NO_CATALOG:
         return "No boot catalog created yet";
+    case ISO_OVWRT_MS_TOO_SMALL:
+        return "Multi-session offset too small for overwrite buffer";
+    case ISO_PART_OFFST_TOO_SMALL:
+        return "Partition offset too small for first tree root.";
+    case ISO_OVWRT_FIFO_TOO_SMALL:
+        return "The ring buffer is too small for overwrite buffer";
     default:
         return "Unknown error";
     }
@@ -426,8 +432,6 @@ int iso_obtain_msgs(char *minimum_severity, int *error_code, int *imgid,
 }
 
 
-/* ts A80222 : derived from libburn/init.c:burn_msgs_submit()
-*/
 int iso_msgs_submit(int error_code, char msg_text[], int os_errno,
 			char severity[], int origin)
 {
@@ -455,8 +459,6 @@ int iso_msgs_submit(int error_code, char msg_text[], int os_errno,
 }
 
 
-/* ts A80222 : derived from libburn/init.c:burn_text_to_sev()
-*/
 int iso_text_to_sev(char *severity_name, int *sevno)
 {
     int ret;
@@ -468,8 +470,6 @@ int iso_text_to_sev(char *severity_name, int *sevno)
 }
 
 
-/* ts A80222 : derived from libburn/init.c:burn_sev_to_text()
-*/
 int iso_sev_to_text(int severity_number, char **severity_name)
 {
     int ret;
@@ -508,7 +508,6 @@ int iso_error_get_code(int e)
 }
 
 
-/* ts A80222 */
 int iso_report_errfile(char *path, int error_code, int os_errno, int flag)
 {
     libiso_msgs_submit(libiso_msgr, 0, error_code,
