@@ -13,6 +13,8 @@
 
    - ExclusionS which manages the list of excluded file paths and
      leaf patterns.
+     Because of its structural identity it is also used for disk address
+     oriented hiding at insert time as of mkisofs.
 
    - Xorriso_lsT which provides a generic double-linked list.
 
@@ -772,7 +774,7 @@ int Exclusions_match(struct ExclusionS *o, char *abs_path, int flag)
 
  /* determine leafname */
  was_non_slash= 0;
- for(leaf_pt= abs_path+strlen(abs_path); leaf_pt>abs_path; leaf_pt--) {
+ for(leaf_pt= abs_path+strlen(abs_path); leaf_pt >= abs_path; leaf_pt--) {
    if(*leaf_pt=='/') {
      if(was_non_slash) {
        leaf_pt++;
@@ -970,7 +972,7 @@ int Permstack_pop(struct PermiteM **o, struct PermiteM *stopper,
      }
    }
    if(!(flag&1)) {
-     chown(m->disk_path, m->stbuf.st_uid, m->stbuf.st_gid);
+     ret= chown(m->disk_path, m->stbuf.st_uid, m->stbuf.st_gid);
                                                /* don't complain if it fails */
      if(!(flag&2)) {
        utime_buffer.actime= m->stbuf.st_atime;
