@@ -125,6 +125,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->loaded_volid[0]= 0;
  m->assert_volid[0]= 0;
  m->assert_volid_sev[0]= 0;
+ m->preparer_id[0]= 0;
  m->publisher[0]= 0;
  m->application_id[0]= 0;
  m->system_id[0]= 0;
@@ -206,6 +207,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->system_area_disk_path[0]= 0;
  m->system_area_options= 0;
  m->patch_system_area= 0;
+ m->partition_offset= 0;
+ m->partition_secs_per_head= 0;
+ m->partition_heads_per_cyl= 0;
  m->vol_creation_time= 0;
  m->vol_modification_time= 0;
  m->vol_expiration_time= 0;
@@ -352,6 +356,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  ret= Xorriso_lst_new(&(m->drive_greylist), "/dev", m->drive_greylist, 1);
  if(ret <= 0)
    goto failure;
+ Xorriso_preparer_string(m, m->preparer_id, 1); /* avoids library calls */
 
  return(1);
 failure:;
