@@ -297,7 +297,7 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
 {
  struct stat s1, s2, stbuf;
  int ret, missing= 0, is_split= 0, i, was_error= 0, diff_count= 0;
- int content_shortcut= 0;
+ int content_shortcut= 0, mask;
  char *respt;
  char a[5*SfileadrL], sfe[5*SfileadrL];
  char ttx1[40], ttx2[40];
@@ -636,8 +636,12 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
  }
  if(was_error)
    ret= -1;
- else
-   ret= (((*result) & ~((1 << 17) | (1 << 18) | (1 << 23)))==0);
+ else {
+   mask= ~((1 << 17) | (1 << 18) | (1 << 22) | (1 << 23));
+   if(xorriso->do_aaip & 32)
+     mask|= 1 << 22;
+   ret= (((*result) & mask)==0);
+ }
 ex:;
  if(split_parts!=NULL)
    Splitparts_destroy(&split_parts, split_count, 0);
