@@ -494,6 +494,110 @@ ex:;
 }
 
 
+/* This function shall know all options of mkisofs, genisoimage, xorrisofs, ...
+   and the number of arguments which they expect and consume.
+*/
+int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
+                                int *count, int flag)
+{
+ int i;
+ char *cmd;
+
+ static char partial_options[][41]= {
+    "errctl=",
+    "isolinux_mbr=", "--modification-date=",
+    ""
+ };
+ static char arg0_options[][41]= {
+    "-allow-leading-dots", "-ldots", "-allow-lowercase", "-allow-multidot",
+    "-cache-inodes", "-no-cache-inodes", "-eltorito-alt-boot",
+    "-hard-disk-boot", "-no-emul-boot", "-no-boot", "-boot-info-table",
+    "-check-oldnames", "-d", "-D", "-dvd-video", "-f", "-gui", "-graft-points",
+    "-hide-joliet-trans-tbl", "-hide-rr-moved", "-J", "-joliet-long", "-l",
+    "-L", "-max-iso9660-filenames", "-N", "-nobak", "-no-bak",
+    "-no-limit-pathtables", "-force-rr", "-no-rr",
+    "-no-split-symlink-components", "-no-split-symlink-fields", "-pad",
+    "-no-pad", "-posix-H", "-posix-L", "-posix-P", "-print-size",
+    "-quiet", "-R", "-r", "-relaxed-filenames", "-rrip110", "-rrip112",
+    "-split-output", "-T", "-UDF", "-udf", "-udf-symlinks", "-no-udf-symlinks",
+    "-U", "-no-iso-translate", "-v", "-XA", "-xa", "-z",
+    "-hfs", "-no-hfs", "-apple", "-probe", "-no-desktop", "-mac-name",
+    "-part", "-icon-position", "-chrp-t", "-hfs-unlock", "--cap", "--netatalk",
+    "--double", "--ethershare", "--ushare", "--exchange", "--sgi", "--xinet",
+    "--macbin", "--single", "--dave", "--sfm", "--osx-double", "--osx-hfs",
+    "-debug", "-omit-period", "-disable-deep-relocation", "-joliet",
+    "-full-iso9660-filenames", "-follow-links", "-help",
+    "-transparent-compression",
+    "-omit-version-number", "-rational-rock", "-rock", "-translation-table",
+    "-untranslated-filenames", "-verbose", "-version", "-g", "-h",
+    "-no-mac-files", "-chrp-boot",
+    "--hardlinks", "--acl", "--xattr", "--md5", "--for_backup",
+    "--protective-msdos-label", "--boot-catalog-hide", "--no-emul-toc",
+    ""
+ };
+ static char arg1_options[][41]= {
+    "-abstract", "-A", "-appid", "-biblio", "-b", "-B", "-boot-load-seg",
+    "-boot-load-size", "-C", "-c", "-check-session", "-copyright",
+    "-dir-mode", "-file-mode", "-G", "-gid", "-hide", "-hide-list",
+    "-hidden", "-hidden-list", "-hide-joliet", "-hide-joliet-list",
+    "-hide-udf", "-hide-udf-list", "-input-charset", "-output-charset",
+    "-iso-level", "-jcharset", "-log-file", "-m", "-exclude-list", "-M",
+    "-dev", "-new-dir-mode", "-o", "-p", "-preparer",
+    "-path-list", "-publisher", "-root",
+    "-old-root", "-s", "-sectype", "-sort", "-sparc-boot", "-sparc-label",
+    "-stream-media-size", "-stream-file-name", "-sunx86-boot", "-sunx86-label",
+    "-sysid", "-table-name", "-ucs-level", "-uid", "-V", "-volset",
+    "-volset-size", "-volset-seqno", "-x", "-P",
+    "-map", "-magic", "-hfs-creator", "-hfs-type", "-boot-hfs-file", "-auto",
+    "-cluster-size", "-hide-hfs", "-hide-hfs-list", "-hfs-volid",
+    "-root-info", "-prep-boot", "-input-hfs-charset", "-output-hfs-charset",
+    "-hfs-bless", "-hfs-parms", 
+    "-eltorito-boot", "-generic-boot", "-eltorito-catalog", "-cdrecord-params",
+    "-errctl", "-exclude", "-prev-session", "-output", "-use-fileversion",
+    "-volid", "-old-exclude",
+    "-alpha-boot", "-hppa-cmdline", "-hppa-kernel-32", "-hppa-kernel-64",
+    "-hppa-bootloader", "-hppa-ramdisk", "-mips-boot", "-mipsel-boot",
+    "-jigdo-jigdo", "-jigdo-template", "-jigdo-min-file-size",
+    "-jigdo-force-md5", "-jigdo-exclude", "-jigdo-map", "-md5-list",
+    "--stdio_sync", "--quoted_path_list", "--efi-boot", "--embedded-boot",
+    "-isohybrid-mbr", "-e", "-partition_offset", "-partition_hd_cyl",
+    "-partition_sec_hd",
+    ""
+ };
+ static char arg2_options[][41]= {
+    "--scdbackup_tag", "--sort-weight",
+    ""
+ };
+ static char final_options[][41]= {
+    "-find",
+    ""
+ };
+
+ cmd= argv[0];
+ *count= 0;
+ for(i=0; partial_options[i][0]!=0; i++)
+   if(strncmp(partial_options[i], cmd, strlen(partial_options[i]))==0)
+     return(1);
+ for(i=0; arg0_options[i][0]!=0; i++)
+   if(strcmp(arg0_options[i], cmd)==0)
+     return(1);
+ *count= 1;
+ for(i=0; arg1_options[i][0]!=0; i++)
+   if(strcmp(arg1_options[i], cmd)==0)
+     return(1);
+ *count= 2;
+ for(i=0; arg2_options[i][0]!=0; i++)
+   if(strcmp(arg2_options[i], cmd)==0)
+     return(1);
+ *count= argc - 1;
+ for(i=0; final_options[i][0]!=0; i++)
+   if(strcmp(final_options[i], cmd)==0)
+     return(1);
+ *count= 0;
+ return(0); 
+}
+
+
 /* @param flag bit0= do not report eventual ignore decision
 */
 int Xorriso_genisofs_ignore(struct XorrisO *xorriso, char *whom,
@@ -509,7 +613,7 @@ int Xorriso_genisofs_ignore(struct XorrisO *xorriso, char *whom,
    ""
  };
  static char ignored_arg1_options[][41]= {
-   "-check-session", "-hide-hfs", "-hide-hfs-list", "-p", "-preparer", "-root",
+   "-check-session", "-hide-hfs", "-hide-hfs-list", "-root",
    "-old-root", "-table-name", "-volset-seqno", "-volset-size",
    ""
  };
@@ -536,23 +640,19 @@ no_volunteer:;
 
 
 int Xorriso_genisofs_add_boot(struct XorrisO *xorriso, char *whom,
-                              int *option_b, int *no_emul_boot, int flag)
+                              int *option_b, int *emul_boot, int flag)
 {
  int ret;
 
- if(*option_b && !*no_emul_boot) {
-   xorriso->boot_image_bin_path[0]= 0;
-   sprintf(xorriso->info_text,
-        "-as %s: Option -b is supported only if option -no-emul-boot is given",
-        whom);
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
- }
+ if(*option_b)
+   xorriso->boot_image_emul= *emul_boot;
+ else
+   xorriso->boot_image_emul= 0;
  ret= Xorriso_attach_boot_image(xorriso, 0);
  if(ret <= 0)
    xorriso->boot_image_bin_path[0]= 0;
  *option_b= 0;
- *no_emul_boot= 1;
+ *emul_boot= 2;
  return(ret);
 }
 
@@ -611,18 +711,24 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -abstract FILE              Set Abstract filename",
 "  -biblio FILE                Set Bibliographic filename",
 "  -copyright FILE             Set Copyright filename",
+"  -p PREP, -preparer PREP     Set Volume preparer",
 "  -b FILE, -eltorito-boot FILE",
 "                              Set El Torito boot image name",
 "  -eltorito-alt-boot          Start specifying alternative El Torito boot parameters",
 "  --efi-boot FILE             Set El Torito EFI boot image name and type",
+"  -e FILE                     Set EFI boot image name (more rawly)",
 "  -c FILE, -eltorito-catalog FILE",
 "                              Set El Torito boot catalog name",
 "  --boot-catalog-hide         Hide boot catalog from ISO9660/RR and Joliet",
 "  -boot-load-size #           Set numbers of load sectors",
+"  -hard-disk-boot             Boot image is a hard disk image",
 "  -no-emul-boot               Boot image is 'no emulation' image",
 "  -boot-info-table            Patch boot image with info table",
 "  -G FILE, -generic-boot FILE Set generic boot image name",
 "  --protective-msdos-label    Patch System Area by partition table",
+"  -partition_offset LBA       Make image mountable by first partition, too",
+"  -partition_sec_hd NUMBER    Define number of sectors per head",
+"  -partition_hd_cyl NUMBER    Define number of heads per cylinder",
 "  --modification-date=YYYYMMDDhhmmsscc",
 "                              Override date of creation and modification",
 "  -isohybrid-mbr FILE         Set SYSLINUX mbr/isohdp[fp]x*.bin for isohybrid",
@@ -738,9 +844,9 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
                      int argc, char **argv, int flag)
 {
  int ret, i, j, was_path= 0, was_other_option= 0, mem_graft_points, mem;
- int do_print_size= 0, fd, idx, iso_level= 0, no_emul_boot= 0;
+ int do_print_size= 0, fd, idx, iso_level= 0, emul_boot= 2;
  int option_b= 0, was_failure= 0, fret, lower_r= 0, zero= 0;
- int dir_mode= -1, file_mode= -1;
+ int dir_mode= -1, file_mode= -1, count;
  mode_t mode_and, mode_or;
  int with_boot_image= 0, with_cat_path= 0;
  int *weight_list= NULL, weight_count= 0;
@@ -907,7 +1013,9 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
      Xorriso_option_stdio_sync(xorriso, argv[i], 0);
    } else
      was_other_option= 1;
-
+     ret= Xorriso_genisofs_count_args(xorriso, argc - i, argv + i, &count, 0);
+     if(ret > 0)
+       i+= count; /* skip eventual arguments of known option */
  continue; /* regular bottom of loop */
 problem_handler_1:;
    was_failure= 1;
@@ -1038,6 +1146,7 @@ not_enough_args:;
      /* was already handled in first argument scan */;
    } else if(strcmp(argv[i], "-V")==0 || strcmp(argv[i], "-volid")==0 ||
              strcmp(argv[i], "-volset")==0 ||
+             strcmp(argv[i], "-p")==0 || strcmp(argv[i], "-preparer")==0 ||
              strcmp(argv[i], "-P")==0 || strcmp(argv[i], "-publisher")==0 ||
              strcmp(argv[i], "-A")==0 || strcmp(argv[i], "-appid")==0 ||
              strcmp(argv[i], "-sysid")==0 ||
@@ -1052,6 +1161,9 @@ not_enough_args:;
        ret= Xorriso_option_volid(xorriso, argv[i], 0);
      else if(strcmp(argv[i - 1], "-volset")==0)
        ret= Xorriso_option_volset_id(xorriso, argv[i], 0);
+     else if(strcmp(argv[i - 1], "-p")==0 ||
+             strcmp(argv[i - 1], "-preparer")==0)
+       ret= Xorriso_option_preparer_id(xorriso, argv[i], 0);
      else if(strcmp(argv[i - 1], "-P")==0 ||
              strcmp(argv[i - 1], "-publisher")==0)
        ret= Xorriso_option_publisher(xorriso, argv[i], 0);
@@ -1092,13 +1204,15 @@ not_enough_args:;
      xorriso->do_disk_pattern= mem;
      if(ret<=0)
        goto problem_handler_2;
-   } else if(strcmp(argv[i], "-v")==0 || strcmp(argv[i], "-quiet")==0) {
+   } else if(strcmp(argv[i], "-v")==0 || strcmp(argv[i], "-verbose")==0 ||
+             strcmp(argv[i], "-quiet")==0) {
      /* was already handled in first argument scan */;
 
    } else if(strcmp(argv[i], "-iso-level")==0) {
      i++;
      /* was already handled in first argument scan */;
    } else if(strcmp(argv[i], "-no-emul-boot")==0 ||
+             strcmp(argv[i], "-hard-disk-boot")==0 ||
              strcmp(argv[i], "-boot-info-table")==0 ||
              strncmp(argv[i], "isolinux_mbr=", 13)==0 ||
              strcmp(argv[i], "-eltorito-alt-boot")==0 ||
@@ -1108,12 +1222,16 @@ not_enough_args:;
    } else if(strcmp(argv[i], "-b") == 0 ||
              strcmp(argv[i], "-eltorito-boot") == 0 ||
              strcmp(argv[i], "--efi-boot") == 0 ||
+             strcmp(argv[i], "-e") == 0 ||
              strcmp(argv[i], "-c") == 0 ||
              strcmp(argv[i], "-eltorito-catalog") == 0 ||
              strcmp(argv[i], "-boot-load-size") == 0 ||
              strcmp(argv[i], "--embedded-boot")==0 ||
              strcmp(argv[i], "-generic-boot")==0 ||
              strcmp(argv[i], "-G") == 0 ||
+             strcmp(argv[i], "-partition_offset") == 0 ||
+             strcmp(argv[i], "-partition_hd_cyl") == 0 ||
+             strcmp(argv[i], "-partition_sec_hd") == 0 ||
              strcmp(argv[i], "-isohybrid-mbr")==0) {
      if(i+1>=argc)
        goto not_enough_args;
@@ -1193,12 +1311,17 @@ not_enough_args:;
        goto problem_handler_2;
      file_mode= mode_or;
    } else if(argv[i][0]=='-' && argv[i][1]!=0) {
-     sprintf(xorriso->info_text, "-as %s: Unknown option %s",
-             whom, Text_shellsafe(argv[i], sfe, 0));
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= Xorriso_genisofs_count_args(xorriso, argc - i, argv + i, &count, 1);
+     if(ret > 0) {
+       sprintf(xorriso->info_text, "-as %s: Unsupported option %s",
+               whom, Text_shellsafe(argv[i], sfe, 0));
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+       i+= count;
+       goto problem_handler_2;
+     } else
+       goto is_pathspec_2;
    } else {
-     int zero= 0;
-
+is_pathspec_2:;
      /* implementing mkisofs tendency to map single-path pathspecs to / */
      if((!xorriso->allow_graft_points) || 
         Fileliste__target_source_limit(argv[i], '=', &ept, 0)<=0) {
@@ -1225,6 +1348,7 @@ not_enough_args:;
        add_pt= argv[i];
      mem_graft_points= xorriso->allow_graft_points;
      xorriso->allow_graft_points= 1;
+     zero= 0;
      ret= Xorriso_option_add(xorriso, 1, &add_pt, &zero, was_path<<1);
      xorriso->allow_graft_points= mem_graft_points;
      if(ret<=0)
@@ -1292,27 +1416,35 @@ problem_handler_2:;
  for(j= 0; j < delay_opt_count; j++) {
    i= delay_opt_list[j];
    if(strcmp(argv[i], "-no-emul-boot")==0) {
-     no_emul_boot= 1;
+     emul_boot= xorriso->boot_image_emul= 0;
+   } else if(strcmp(argv[i], "-hard-disk-boot")==0) {
+     emul_boot= xorriso->boot_image_emul= 1;
    } else if(strcmp(argv[i], "-boot-info-table")==0) {
      xorriso->patch_isolinux_image= 1;
    } else if(strcmp(argv[i], "-b") == 0 ||
              strcmp(argv[i], "-eltorito-boot") == 0 ||
-             strcmp(argv[i], "--efi-boot") == 0) {
+             strcmp(argv[i], "--efi-boot") == 0 ||
+             strcmp(argv[i], "-e") == 0) {
      i++;
      if(strcmp(argv[i - 1], "--efi-boot") == 0) {
        if(xorriso->boot_image_bin_path[0]) {
          ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                        &option_b, &no_emul_boot, 0);
+                                        &option_b, &emul_boot, 0);
          if(ret <= 0)
            goto problem_handler_boot;
        }
        boot_path= xorriso->boot_image_bin_path;
        xorriso->boot_efi_default= 1;
+       emul_boot= xorriso->boot_image_emul= 0;
      } else {
        boot_path= xorriso->boot_image_bin_path; 
-       xorriso->boot_platform_id= 0x00;
+       if(strcmp(argv[i - 1], "-e") == 0)
+         xorriso->boot_platform_id= 0xef;
+       else
+         xorriso->boot_platform_id= 0x00;
        xorriso->boot_efi_default= 0;
        option_b= 1;
+       xorriso->boot_image_emul= emul_boot;
      }
      boot_path[0]= 0;
      if(argv[i][0] != '/')
@@ -1322,8 +1454,9 @@ problem_handler_2:;
        goto ex;
      if(xorriso->boot_efi_default && xorriso->boot_image_bin_path[0]) {
        option_b= 0;
+       emul_boot= 0;
        ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                      &option_b, &no_emul_boot, 0);
+                                      &option_b, &emul_boot, 0);
        if(ret <= 0)
          goto problem_handler_boot;
      }
@@ -1356,7 +1489,7 @@ problem_handler_2:;
        goto problem_handler_boot;
    } else if(strcmp(argv[i], "-eltorito-alt-boot")==0) {
      ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                    &option_b, &no_emul_boot, 0);
+                                    &option_b, &emul_boot, 0);
      if(ret <= 0)
        goto problem_handler_boot;
    } else if(strcmp(argv[i], "--embedded-boot")==0 ||
@@ -1375,6 +1508,16 @@ problem_handler_2:;
      xorriso->system_area_options= (xorriso->system_area_options & ~3) | 1;
    } else if(strcmp(argv[i], "--boot-catalog-hide")==0) {
      xorriso->boot_image_cat_hidden|= 3;
+   } else if(strcmp(argv[i], "-partition_offset") == 0 ||
+             strcmp(argv[i], "-partition_sec_hd") == 0 ||
+             strcmp(argv[i], "-partition_dh_cyl") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     sprintf(sfe, "%s=%.16s", argv[i-1] + 1, argv[i]);
+     ret= Xorriso_option_boot_image(xorriso, "any", sfe, 0);
+     if(ret <= 0)
+       goto problem_handler_boot;
    }
  continue; /* regular bottom of loop */
 problem_handler_boot:;
@@ -1388,7 +1531,7 @@ problem_handler_boot:;
  if(with_boot_image && with_cat_path == 0)
    strcpy(xorriso->boot_image_cat_path, "/boot.catalog");
  if(xorriso->boot_image_bin_path[0]) {
-   ret= Xorriso_genisofs_add_boot(xorriso, whom, &option_b, &no_emul_boot, 0);
+   ret= Xorriso_genisofs_add_boot(xorriso, whom, &option_b, &emul_boot, 0);
    if(ret <= 0)
      goto ex;
  }
