@@ -1313,6 +1313,27 @@ ex:;
 }
 
 
+int Xorriso_node_eff_hidden(struct XorrisO *xorriso, IsoNode *node, int flag)
+{
+ int hidden_state= 0, ret;
+ IsoNode *current, *parent;
+
+ current= node;
+ for(current= node; hidden_state != 3;) {
+   ret= iso_node_get_hidden(current);
+   if(ret & LIBISO_HIDE_ON_RR)
+     hidden_state|= 1;
+   if(ret & LIBISO_HIDE_ON_JOLIET)
+     hidden_state|= 2;
+   parent= (IsoNode *) iso_node_get_parent(current);
+   if(parent == current)
+ break;
+   current= parent;
+ }
+ return(hidden_state);
+}
+
+
 /* @param flag bit0= long format
                bit1= do not print count of nodes
                bit2= du format
@@ -1321,7 +1342,7 @@ ex:;
 int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
                      int filec, char **filev, off_t boss_mem, int flag)
 {
- int i, ret, was_error= 0, dfilec= 0, pass, passes;
+ int i, ret, was_error= 0, dfilec= 0, pass, passes, hidden_state= 0;
  IsoNode *node;
  IsoImage *volume;
  char sfe[5*SfileadrL], sfe2[5*SfileadrL], path[SfileadrL];
@@ -1404,8 +1425,10 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
    link_target[0]= 0;
    if((flag&5)==1) { /* -ls_l */
      iso_node_get_acl_text(node, &a_text, &d_text, 16);
+     hidden_state= Xorriso_node_eff_hidden(xorriso, node, 0);
      ret= Xorriso_format_ls_l(xorriso, &stbuf,
-                              1 | ((a_text != NULL || d_text != NULL) << 1));
+                              1 | ((a_text != NULL || d_text != NULL) << 1) |
+                              (hidden_state << 2));
      iso_node_get_acl_text(node, &a_text, &d_text, 1 << 15);
      if(ret<=0)
  continue;
@@ -1835,6 +1858,46 @@ int Xorriso__file_start_lba(IsoNode *node,
 }
 
 
+int Xorriso_retrieve_disk_path(struct XorrisO *xorriso, IsoNode *node,
+                               char disk_path[SfileadrL], int flag)
+{
+ IsoFile *file;
+ IsoStream *stream= NULL, *input_stream;
+ char type_text[80], *source_path = NULL;
+
+ if(!LIBISO_ISREG(node)) 
+   return(0);
+
+ /* Obtain most fundamental input stream */
+ file= (IsoFile *) node;
+ input_stream= iso_file_get_stream(file);
+ if(input_stream == NULL)
+   return(0);
+ while(1) {
+   stream= input_stream;
+   input_stream= iso_stream_get_input_stream(stream, 0);
+   if(input_stream == NULL)
+ break;
+ }
+
+ /* Obtain disk path if applicable */
+ type_text[0]= 0;
+ Xorriso_stream_type(xorriso, node, stream, type_text, 0);
+ if(strcmp(type_text, "disk") != 0 && strcmp(type_text, "cout") != 0)
+   return(0); /* among othersi rejected: "image" */
+ source_path= iso_stream_get_source_path(stream, 0);
+ if(source_path == NULL)
+   return(0);
+ if(strlen(source_path) >= SfileadrL) {
+   free(source_path);
+   return(0);
+ }
+ strcpy(disk_path, source_path);
+ free(source_path);
+ return(1);
+}
+
+
 int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
                         char *path, int flag)
 {
