@@ -181,6 +181,10 @@ LIBISOBURN_MISCONFIGURATION_ = 0;
    xorriso->zlib_level= xorriso->zlib_level_default=
        zisofs_ctrl.compression_level;
  }
+
+ /* Second initialization. This time with libs. */
+ Xorriso_preparer_string(xorriso, xorriso->preparer_id, 0);
+
  Xorriso_process_msg_queues(xorriso,0);
  if(reason[0]) {
    sprintf(xorriso->info_text, "%s", reason);
@@ -486,3 +490,34 @@ int Xorriso_md5_end(struct XorrisO *xorriso, void **ctx, char md5[16],
  return(1);
 }
 
+
+/* @param flag  bit0= avoid library calls
+ */
+int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
+                            int flag)
+{
+ int major, minor, micro;
+
+ xorriso_id[0]= 0;
+ sprintf(xorriso_id, "XORRISO-%d.%d.%d",
+         Xorriso_header_version_majoR, Xorriso_header_version_minoR,
+         Xorriso_header_version_micrO);
+ if(strlen(xorriso_id) + strlen(Xorriso_timestamP) < 128)
+   strcat(xorriso_id, Xorriso_timestamP);
+ if(flag & 1)
+   return(1);
+ isoburn_version(&major, &minor, &micro);
+ if(strlen(xorriso_id) < 100)
+   sprintf(xorriso_id + strlen(xorriso_id),
+           ", LIBISOBURN-%d.%d.%d", major, minor, micro);
+ iso_lib_version(&major, &minor, &micro);
+ if(strlen(xorriso_id) < 100)
+   sprintf(xorriso_id + strlen(xorriso_id),
+           ", LIBISOFS-%d.%d.%d", major, minor, micro);
+ burn_version(&major, &minor, &micro);
+ if(strlen(xorriso_id) < 100)
+   sprintf(xorriso_id + strlen(xorriso_id),
+           ", LIBBURN-%d.%d.%d", major, minor, micro);
+ return(1);
+}
+
