@@ -259,11 +259,14 @@ ex:;
 int Xorriso_option_application_id(struct XorrisO *xorriso, char *name,
                                   int flag)
 {
-  if(Xorriso_check_name_len(xorriso, name,
-                            (int) sizeof(xorriso->application_id),
-                            "-application_id", 0) <= 0)
-    return(0);
- strcpy(xorriso->application_id,name);
+ if(Xorriso_check_name_len(xorriso, name,
+                           (int) sizeof(xorriso->application_id),
+                           "-application_id", 0) <= 0)
+   return(0);
+ if(strcmp(name, "@xorriso@") == 0)
+   Xorriso_preparer_string(xorriso, xorriso->application_id, 0);
+ else
+   strcpy(xorriso->application_id,name);
  Xorriso_set_change_pending(xorriso, 1);
  return(1);
 }
@@ -788,6 +791,40 @@ treatment_patch:;
    } else
      was_ok= 0;
 
+ } else if(strncmp(treatpt, "partition_offset=", 17)==0) {
+   u= 0;
+   sscanf(treatpt + 17, "%u", &u);
+   if(u > 0 && u < 16) {
+     sprintf(xorriso->info_text,
+      "-boot_image %s partition_offset= : Non-zero number too small (<16).",
+             formpt);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+   xorriso->partition_offset= u;
+
+ } else if(strncmp(treatpt, "partition_hd_cyl=", 17)==0) {
+   u= 0;
+   sscanf(treatpt + 17, "%u", &u);
+   if(u > 255) {
+     sprintf(xorriso->info_text,
+      "-boot_image %s partition_hd_cyl= : Number too large (>255).", formpt);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+   xorriso->partition_heads_per_cyl= u;
+
+ } else if(strncmp(treatpt, "partition_sec_hd=", 17)==0) {
+   u= 0;
+   sscanf(treatpt + 17, "%u", &u);
+   if(u > 63) {
+     sprintf(xorriso->info_text,
+      "-boot_image %s partition_sec_hd= : Number too large (>63).", formpt);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+   xorriso->partition_secs_per_head= u;
+
  } else if(strncmp(treatpt, "platform_id=", 12)==0) {
    if(strncmp(treatpt + 12, "0x", 2) == 0)
      sscanf(treatpt + 14, "%x", &u);
@@ -802,6 +839,23 @@ treatment_patch:;
    }
    xorriso->boot_platform_id= u;
    
+ } else if(strncmp(treatpt, "emul_type=", 10)==0) {
+   if(strcmp(treatpt + 10, "none") == 0 ||
+      strcmp(treatpt + 10, "no_emulation") == 0) {
+     xorriso->boot_image_emul= 0;
+   } else if(strcmp(treatpt + 10, "hard_disk") == 0) {
+     xorriso->boot_image_emul= 1;
+   } else if(strcmp(treatpt + 10, "floppy") == 0 ||
+             strcmp(treatpt + 10, "diskette") == 0) {
+     xorriso->boot_image_emul= 2;
+   } else {
+     sprintf(xorriso->info_text,
+             "-boot_image %s : Unknown media_type : %s",
+             formpt, treatpt + 10);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+
  } else if(strncmp(treatpt, "isohybrid=", 10) == 0 &&
            strcmp(formpt, "isolinux")==0) {
 
