@@ -218,6 +218,22 @@ int Xorriso_option_pkt_output(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
+/* Option -preparer_id */
+int Xorriso_option_preparer_id(struct XorrisO *xorriso, char *name, int flag)
+{
+ if(Xorriso_check_name_len(xorriso, name,
+                           (int) sizeof(xorriso->preparer_id),
+                           "-preparer_id", 0) <= 0)
+   return(0);
+ if(strcmp(name, "@xorriso@") == 0)
+   Xorriso_preparer_string(xorriso, xorriso->preparer_id, 0);
+ else
+   strcpy(xorriso->preparer_id, name);
+ Xorriso_set_change_pending(xorriso, 1);
+ return(1);
+}
+
+
 /* Option -print */
 int Xorriso_option_print(struct XorrisO *xorriso, char *text, int flag)
 {
