@@ -465,7 +465,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "list_delimiter","list_profiles","local_charset",
     "mark","md5","mount_opts","not_leaf","not_list","not_mgt",
     "options_from_file","osirrox","outdev","out_charset","overwrite",
-    "pacifier","padding","path_list","pathspecs","pkt_output","print","prompt",
+    "pacifier","padding","path_list","pathspecs","pkt_output",
+    "preparer_id","print","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
     "reassure","report_about","rom_toc_scan","scsi_log",
     "session_log","speed","split_size","status","status_history_max",
@@ -1114,6 +1115,10 @@ next_command:;
    (*idx)++;
    ret=  Xorriso_option_pkt_output(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"preparer_id")==0) {
+   (*idx)++;
+   ret= Xorriso_option_preparer_id(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"print")==0) {
    (*idx)++;
    ret= Xorriso_option_print(xorriso, arg1, 0);
