@@ -213,6 +213,8 @@ int Xorriso_set_system_area(struct XorrisO *xorriso, struct burn_drive *drive,
  unsigned char *ub;
  ElToritoBootImage *bootimg;
  IsoFile *bootimg_node;
+ uint32_t offst;
+ enum burn_disc_status state;
 
  system_area_options= xorriso->system_area_options;
  memset(buf, 0, 32768);
@@ -308,6 +310,27 @@ do_set:;
                       0, "FAILURE", 1);
    {ret= 0; goto ex;}
  }
+ offst= xorriso->partition_offset;
+ state= isoburn_disc_get_status(drive);
+ if(state == BURN_DISC_APPENDABLE) {
+   ret= isoburn_get_img_partition_offset(drive, &offst);
+   if(ret == 1) {
+     sprintf(xorriso->info_text,
+             "Preserving in ISO image: -boot_image any partition_offset=%lu",
+             (unsigned long) offst);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+   } else
+     offst= xorriso->partition_offset;
+ }
+ ret= isoburn_igopt_set_part_offset(sopts, offst,
+                                    xorriso->partition_secs_per_head,
+                                    xorriso->partition_heads_per_cyl);
+ if(ret != ISO_SUCCESS) {
+   Xorriso_process_msg_queues(xorriso,0);
+   Xorriso_report_iso_error(xorriso, "", ret,
+                       "Error when setting partition offset", 0, "FAILURE", 1);
+   {ret= 0; goto ex;}
+ }
  ret= 1;
 ex:;
  if(fp != NULL && fp != stdin)
@@ -487,7 +510,7 @@ ex:;
 int Xorriso_write_session(struct XorrisO *xorriso, int flag)
 {
  int ret, relax= 0, i, pacifier_speed= 0, data_lba, ext, is_bootable= 0;
- int major, minor, micro, freshly_bootable= 0, hide_attr;
+ int freshly_bootable= 0, hide_attr;
  char xorriso_id[256], *img_id, sfe[5*SfileadrL], *out_cs;
  struct isoburn_imgen_opts *sopts= NULL;
  struct burn_drive_info *dinfo, *source_dinfo;
@@ -522,6 +545,7 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
                                 "on attempt to write", 2);
  if(ret<=0)
    return(0);
+
  if(xorriso->out_drive_handle == xorriso->in_drive_handle) {
    source_drive= drive;
  } else {
@@ -692,22 +716,7 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
                                  xorriso->scdbackup_tag_time,
                                  xorriso->scdbackup_tag_written);
  if(image!=NULL && 12+strlen(Xorriso_timestamP)<80) {
-   sprintf(xorriso_id, "XORRISO-%d.%d.%d %s",
-           Xorriso_header_version_majoR, Xorriso_header_version_minoR,
-           Xorriso_header_version_micrO, Xorriso_timestamP);
-   isoburn_version(&major, &minor, &micro);
-   if(strlen(xorriso_id)<80)
-     sprintf(xorriso_id+strlen(xorriso_id),
-             ", LIBISOBURN-%d.%d.%d", major, minor, micro);
-   iso_lib_version(&major, &minor, &micro);
-   if(strlen(xorriso_id)<80)
-     sprintf(xorriso_id+strlen(xorriso_id),
-             ", LIBISOFS-%d.%d.%d", major, minor, micro);
-   burn_version(&major, &minor, &micro);
-   if(strlen(xorriso_id)<80)
-     sprintf(xorriso_id+strlen(xorriso_id),
-             ", LIBBURN-%d.%d.%d", major, minor, micro);
-   xorriso_id[128]= 0;
+   strcpy(xorriso_id, xorriso->preparer_id);
    img_id= (char *) iso_image_get_data_preparer_id(image);
    if(img_id!=NULL) {
      for(i= strlen(img_id)-1; i>=0 && img_id[i]==' '; i--);
