@@ -19,6 +19,10 @@
 #define Xorriso_private_includeD yes
 
 
+/* for uint32_t */
+#include <stdint.h>
+
+
 /** The source code release timestamp */
 #include "xorriso_timestamp.h"
 #ifndef Xorriso_timestamP
@@ -80,6 +84,9 @@ struct XorrisO { /* the global context of xorriso */
  char rc_filenames[Xorriso_rc_nuM][SfileadrL];
  int rc_filename_count;
 
+ /* Whether .mkisofsrc has already been read */
+ int mkisofsrc_done;
+
  char wdi[SfileadrL];
  char wdx[SfileadrL];
  int did_something_useful;
@@ -166,11 +173,17 @@ struct XorrisO { /* the global context of xorriso */
  char assert_volid[SfileadrL];
  char assert_volid_sev[80];
 
+ char preparer_id[129];
+
  char publisher[129];
  char application_id[129];
  char system_id[33];
  char volset_id[129];
 
+ char copyright_file[38];
+ char biblio_file[38];
+ char abstract_file[38];
+
  char session_logfile[SfileadrL];
  int session_lba;
  int session_blocks;
@@ -211,8 +224,8 @@ struct XorrisO { /* the global context of xorriso */
  int do_calm_drive;      /* bit0= calm down drive after aquiring it */
 
  char indev[SfileadrL];
- void *in_drive_handle;  /* interpreted only by xorrisoburn.c */
- void *in_volset_handle; /* interpreted only by xorrisoburn.c */
+ void *in_drive_handle;  /* interpreted only by libburnia oriented modules */
+ void *in_volset_handle; /* interpreted only by libburnia oriented modules */
  char *in_charset;       /* The charset to interpret the filename bytes */
  int indev_is_exclusive;
 
@@ -262,6 +275,7 @@ struct XorrisO { /* the global context of xorriso */
 
  int keep_boot_image;
  char boot_image_cat_path[SfileadrL];
+ int boot_image_cat_hidden;   /* bit0= hidden in ISO/RR , bit1= in Joliet */
  int boot_count; /* number of already attached boot images */
 
  char boot_image_bin_path[SfileadrL];
@@ -269,8 +283,8 @@ struct XorrisO { /* the global context of xorriso */
  int boot_platform_id; 
  int patch_isolinux_image; /* bit0= boot-info-table , bit1= not with EFI */
  int boot_image_emul; /* 0=no emulation
-                        (1=emulation as hard disk)
-                        (2=emulation as floppy)
+                         1=emulation as hard disk
+                         2=emulation as floppy
                       */
  off_t boot_image_load_size;
  unsigned char boot_id_string[29];
@@ -292,6 +306,17 @@ struct XorrisO { /* the global context of xorriso */
                                          area of the image, if no 
                                          system_area_disk_path is set.
                                        */
+
+ /* The number of unclaimed 2K blocks before start of partition 1 as of
+    the MBR in system area.
+    If not 0 this will cause double volume descriptor sets and double tree.
+ */
+ uint32_t partition_offset;
+ /* Partition table parameter: 1 to 63, 0= disabled/default */
+ int partition_secs_per_head;
+ /* 1 to 255, 0= disabled/default */
+ int partition_heads_per_cyl;
+
  /* User settable PVD time stamps */
  time_t vol_creation_time;
  time_t vol_modification_time;
@@ -379,6 +404,9 @@ struct XorrisO { /* the global context of xorriso */
                               than to treat them as truely missing on disk
                      */
 
+ struct ExclusionS *iso_rr_hidings;
+ struct ExclusionS *joliet_hidings;
+
  int use_stdin; /* use raw stdin even if readline support is compiled */
  int result_page_length;
  int result_page_width;
