@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  0
 #define xorriso_libisoburn_req_minor  6
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  2
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -534,5 +534,11 @@ int Xorriso_set_system_area_path(struct XorrisO *xorriso, char *path,
 int Xorriso_set_hidden(struct XorrisO *xorriso, void *in_node, char *path,
                        int hide_state, int flag);
 
+
+/* @param flag  bit0= avoid library calls
+ */
+int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
+                            int flag);
+
 #endif /* Xorrisoburn_includeD */
 
